/*
 * This file is part of SpongeAPI, licensed under the MIT License (MIT).
 *
 * Copyright (c) SpongePowered <https://www.spongepowered.org>
 * Copyright (c) contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package org.spongepowered.api.entity.living.monster;

import org.spongepowered.api.data.key.Keys;
import org.spongepowered.api.data.manipulator.mutable.entity.ElderData;
import org.spongepowered.api.data.value.mutable.Value;
import org.spongepowered.api.entity.living.Aquatic;
import org.spongepowered.api.entity.living.Living;

import java.util.Optional;

/**
 * Represents a Guardian.
 */
public interface Guardian extends Aquatic, Monster {

    /**
     * Gets the {@link ElderData} for the "elder" state.
     *
     * @return The elder data manipulator
     */
    default ElderData getElderData() {
        return get(ElderData.class).get();
    }

    /**
     * Gets the {@link Value} for the "elder" state.
     *
     * @return The value for the elder state
     */
    default Value<Boolean> elder() {
        return getValue(Keys.ELDER_GUARDIAN).get();
    }

    /**
     * Gets the target of the guardian's beam or {@link Optional#empty()} otherwise.
     *
     * @return An optional containing the target, if any
     */
    Optional<Living> getBeamTarget();

    /**
     * Sets the target of the guardian's beam. Setting a null value removes the target.
     *
     * @param entity The entity or null to clear it
     */
    void setBeamTarget(Living entity);
}
